//=============================================================================
// Ammunition System, ver0.2.2
//   by Unconnected42
// UNCO_AmmunitionSystem.js
// Last Updated : 2015/11/08
//=============================================================================

var Imported = Imported || {};
Imported.UNCO_AmmunitionSystem = true;

var Unco = Unco || {};
Unco.AS = Unco.AS || {};


//=============================================================================
 /*:
 * @plugindesc  Allows skills to need ammunition.
 * <Unco Ammo>
 * @author Unconnected42
 *
 * @param Show Ammo Left When Actor Command
 * @desc If set as 'true', show how much ammunition is left for main attack at the top of the screen, during actor command selection.
 * @default true
 *
 * @param Show Ammo Left In Help Window
 * @desc  If set as 'true', show how much ammunition is left for a skill in its description.
 * @default true
 *
 * @param Help Window Ammo Text
 * @desc  Default text to be displayed in skill description before ammunition amounts.
 * @default Ammunition left
 *
 * @param Show Ammo Left In Skill Window
 * @desc In skill window, show how much ammunition is left together with the cost if set at 'true'.
 * @default false
 *
 * @param No Ammo Icon
 * @desc  Id of the default icon to be displayed when there is no more ammunition and several types of ammunition are possible.
 * @default 16
 *
 * @param Ammo Font Size
 * @desc Font size of the ammunition amounts in skill window and gauge.
 * @default 20
 *
 * @param Ammo Font Color
 * @desc Font color of the ammunition amounts in skill window and gauge.
 * @default 6
 *
 * @param Show Ammo Left In Actor Command Window
 * @desc If set at true, will display how much ammo is left just next to the attack command.
 * @default false
 *
 * @help
 * ============================================
 * Introduction
 * ============================================
 * 
 * This plug-in will allow skills or weapons to require the consumption of 
 * ammunition, components, etc. in order to work. The cost is  paid each 
 * time the skill is used, together with MP/TP costs, or each time the 
 * normal attack is used in the case of weapons.
 * The ammunition can be either gold, items or equipments (armours), or a mix.
 * 
 * However, things work differently depending on what type of cost is considered.
 * - items : if several types of items are marked as required as ammunition 
 *   for a given skill/weapon, *ALL* of the corresponding prices must be 
 *   paid each time the skill/weapon is used. 
 * - equipment : the equipment needs to be equipped in order for the skill/weapon
 *   to work. If several types of equipments are marked as required as ammunition
 *   for a given skill/weapon, *ONLY* the cost of the one that is *CURRENTLY
 *   EQUIPPED* will be paid. That means a weapon/skill can be able to use 
 *   different interchangeable ammunition types.
 * - Gold cost is straightforward in how it functions.
 * 
 * ============================================
 * Known Compatibility Issues
 * ============================================
 * 
 * This plug-in should be placed under all plug-ins that you are using,
 * especially Yanfly's since it actually uses some of their features if
 * they are present.
 * Known exceptions to that rule : 
 * - Yanfly's Skill Cooldown
 * - Jay's Dual Techs. 
 *   -> Ammunition should be placed above these two plug-ins.
 * On the contrary, the following plug-ins MUST be placed above :
 * - Bobstah's BattleCommandList.
 * 
 * This might also be the case for other plug-ins whenever 
 * the modification of skill costs is concerned.
 *
 * This plug-in in the present version should be fully compatible
 * with Yanfly's ItemCore Independent Items feature.
 * Therefore, ammunition of any kind can either be independent or
 * not (for the latter case, use a <Not Independent Item> notetag
 * in the ammunition notebox if you are using ItemCore and a max
 * number of items > 0).
 * 
 * ============================================
 * Use
 * ============================================
 * 
 * Lines to put in either Weapon notebox or Skill notebox :
 * <Gold Cost: c>
 * <Ammo i Item: c>  (for items)
 * <Ammo i Equip: c>  (for armors)
 * ... where i is the ID of the ammunition item/armor and c is the cost 
 * required for one use.
 * 
 * ============================================
 * Displaying Ammunition Amounts & Costs
 * ============================================
 * 
 * A given ammunition type is almost always represented by its icon when it
 * is necessary (either to display a cost or a remaining amount).
 * Ammo costs for skills are indicated together with "normal" TP/MP costs.
 * 
 * There are several solutions for displaying the remaining ammunition amount
 * so that the player knows where he stands.
 * Most of them are turned on through the plug-in parameters.
 * 
 * - For weapons : 
 *   + The number of ammunition left can be displayed at the
 *     top of the screen when the actor command menu is active.
 *     This is the default option.
 *   + The number of ammunition left can be displayed in the 
 *     command window, just next to the "Attack" option.
 *     No icon in that case.
 *   + If you are using Yanfly's SkillCore plug-in, you can use the following
 *     notetag in the weapon note :
 *         <Swap Gauge n: AMMO>
 *     ...where n is the number of the gauge you want to replace.
 *     Instead of a normal HP/MP/TP gauge, you will then have the amount of
 *     ammunition left for the currently equipped weapon.
 * - For skills : 
 *   + The remaining amount for each ammunition type can be showed
 *     together with cost in the skill window.
 *   + It can also be displayed after the skill description, in the help
 *     window. This is the default option.
 *     In that case, you should keep the skill description within one unique
 *     line with no line-break at the end, or else the remaining ammo will 
 *     not be visible.
 * 
 * The text before ammunition left amounts can be customized with the following
 *   notetag in the skill notebox:
 *   <Ammo Left Text: xxx>
 *   ... with xxx being your custom text.
 * 
 * Also, in the case of equipment ammunition, several types of
 * ammunition are possible but only one (the one equipped) will be displayed
 * in any skill/help/etc window (or gauge). When there is no more ammunition,
 * the system normally would not know which icon to use !
 * There is a default icon for that, but you can put the following notetag
 * in skill/weapon notebox to define which icon will be used :
 *   <No Ammo Icon: i>
 *   ... with i being the icon number. 
 * It is also possible to add details about the ammunition type currently 
 * equipped by adding the following notetag in the ammo notebox :
 *   <Ammo Window Desc: xxx>
 * Where 'xxx' is the description you want to give to your ammo.
 * This description will show up in the skill help window and the ammunition
 * window visible during actor command selection.
 * Please that if you deactivate the ammo window or the display of remaining
 * ammo amount with the corresponding plug-in parameters, you will of course 
 * not see any description of ammunition !
 */
//=============================================================================

//=============================================================================
// Parameter Variables
//=============================================================================

Unco.Parameters = $plugins.filter(function(p) {
        return p.description.contains('<Unco Ammo>');
    })[0].parameters; //Copied from Ellye, who thanks Iavra
Unco.Param = Unco.Param || {};

Unco.Param.showAmmoLeftActorCommand = String(Unco.Parameters['Show Ammo Left When Actor Command']).toLowerCase();
Unco.Param.showAmmoLeft = String(Unco.Parameters['Show Ammo Left In Skill Window']).toLowerCase();
Unco.Param.ammoFontSize = Number(Unco.Parameters['Ammo Font Size']);
Unco.Param.ammoFontColor = Number(Unco.Parameters['Ammo Font Color']);
Unco.Param.showAmmoLeftInDesc = String(Unco.Parameters['Show Ammo Left In Help Window']).toLowerCase();
Unco.Param.descAmmoLeftText = String(Unco.Parameters['Help Window Ammo Text']);
Unco.Param.defaultNoAmmoIconId = parseInt(String(Unco.Parameters['No Ammo Icon']));
Unco.Param.showAmmoLeftForAttack = String(Unco.Parameters['Show Ammo Left In Actor Command Window']).toLowerCase();

//=============================================================================
// DataManager
//=============================================================================

Unco.AS.DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
DataManager.isDatabaseLoaded = function() {
    if (!Unco.AS.DataManager_isDatabaseLoaded.call(this)) return false;
    this.processUncoItemAmmoNotetags($dataSkills);
    this.processUncoGoldCostNotetags($dataSkills);
    this.processUncoAmmoTextNotetags($dataSkills);
    this.processUncoEquipAmmoNotetags($dataSkills);
    this.processUncoAmmoTextNotetags($dataWeapons);
    this.processUncoItemAmmoNotetags($dataWeapons);
    this.processUncoEquipAmmoNotetags($dataWeapons);
    this.processUncoAmmoDescNotetags($dataArmors);
    return true;
};

DataManager.processUncoGoldCostNotetags = function(group) {
   for (var n = 1; n < group.length; n++) {
      var obj = group[n];
      var notedata = obj.note.split(/[\r\n]+/);

      for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:GOLD)[ ](?:COST):[ ](\d+)>/i)) {
            var value = parseInt(RegExp.$1);
            if (!isNaN(value)) {
               obj.goldCost = value;
            }
         }
      }
   }
}

DataManager.processUncoItemAmmoNotetags = function(group) {
   for (var n = 1; n < group.length; n++) {
      var obj = group[n];
      var notedata = obj.note.split(/[\r\n]+/);

      obj.itemAmmoCost = [];

      for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:AMMO)[ ](\d+)[ ](?:ITEM):[ ](\d+)>/i)) {
            var index = parseInt(RegExp.$1);
            var value = parseInt(RegExp.$2);
            if ( (!isNaN(index)) && (!isNaN(value)) ) {
               obj.itemAmmoCost[index] = value;
               if (DataManager.isWeapon(obj)) {
                  if (typeof $dataItems[index].isItemAmmoForWeapon === 'undefined') $dataItems[index].isItemAmmoForWeapon = [];
                  $dataItems[index].isItemAmmoForWeapon[obj.id] = value;
               } else {
                  if (typeof $dataItems[index].isItemAmmoForSkill === 'undefined') $dataItems[index].isItemAmmoForSkill = [];
                  $dataItems[index].isItemAmmoForSkill[obj.id] = value;
               }
            }
         }
      }
   }
}

DataManager.processUncoEquipAmmoNotetags = function(group) {
   for (var n = 1; n < group.length; n++) {
      var obj = group[n];
      var notedata = obj.note.split(/[\r\n]+/);

      obj.equipAmmoCost = [];

      for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:AMMO)[ ](\d+)[ ](?:EQUIP):[ ](\d+)>/i)) {
            var index = parseInt(RegExp.$1);
            var value = parseInt(RegExp.$2);
            if ( (!isNaN(index)) && (!isNaN(value)) ) {
               obj.equipAmmoCost[index] = value;
               if (DataManager.isWeapon(obj)) {
                  if (typeof $dataArmors[index].isEquipAmmoForWeapon === 'undefined') $dataArmors[index].isEquipAmmoForWeapon = [];
                  $dataArmors[index].isEquipAmmoForWeapon[obj.id] = value;
               } else {
                  if (typeof $dataArmors[index].isEquipAmmoForSkill === 'undefined') $dataArmors[index].isEquipAmmoForSkill = [];
                  $dataArmors[index].isEquipAmmoForSkill[obj.id] = value;
               }
            }
         }
      }
   }
}

DataManager.isItemAmmo = function(item) {
   return this.isItem(item) && (item.isItemAmmoForWeapon || item.isItemAmmoForSkill);
};
DataManager.isEquipAmmo = function(item) {
   return this.isArmor(item) && (item.isEquipAmmoForWeapon || item.isEquipAmmoForSkill);
};
DataManager.isAmmo = function(item) {
   return (this.isItemAmmo(item) || this.isEquipAmmo(item));
};

DataManager.processUncoAmmoTextNotetags = function(group) {
   for (var n = 1; n < group.length; n++) {
      var obj = group[n];
      var notedata = obj.note.split(/[\r\n]+/);
      for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:AMMO)[ ](?:LEFT)[ ](?:TEXT):[ ](.*)>/i)) {
            obj.ammoText = String(RegExp.$1);
         }
         if (line.match(/<(?:NO)[ ](?:AMMO)[ ](?:ICON):[ ](\d+)>/i)) {
            obj.noAmmoIconId = parseInt(RegExp.$1);
         }
      }
   }
};

DataManager.processUncoAmmoDescNotetags = function(group) {
   for (var n = 1; n < group.length; n++) {
      var obj = group[n];
      var notedata = obj.note.split(/[\r\n]+/);
      for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:AMMO)[ ](?:WINDOW)[ ](?:DESC):[ ](.*)>/i)) {
            obj.ammoDesc = String(RegExp.$1);
         }
      }
   }
};

//=============================================================================
// Game_BattlerBase
//=============================================================================

Unco.AS.Game_BattlerBase_canEquipArmor = Game_BattlerBase.prototype.canEquipArmor;
Game_BattlerBase.prototype.canEquipArmor = function(item) {
   if (typeof $dataArmors[item.id].isEquipAmmoForWeapon !== 'undefined') {
      var ok = false;
		if (this._equips[0].itemId() > 0) {
		   var EquippedWeaponId = ( (typeof $dataWeapons[this._equips[0].itemId()].baseItemId === 'undefined') ? this._equips[0].itemId() : $dataWeapons[this._equips[0].itemId()].baseItemId );
		   for (var weaponId in $dataArmors[item.id].isEquipAmmoForWeapon) {
		      weaponId = parseInt(weaponId);
            if (weaponId === EquippedWeaponId) {
               ok = true;
				   break;
            }
         }
		}
      if (ok === false) return false;
   }
   return Unco.AS.Game_BattlerBase_canEquipArmor.call(this,item);
};

Game_Party.prototype.getItemAmount = function(baseItem) {
   if (!baseItem) return 0;
   var amount = 0;
   if (Imported.YEP_ItemCore) { 
      var maxItems = 0;
      if (DataManager.isItem(baseItem)) {
         maxItems = Yanfly.Param.ItemMaxItems;
      }
      if (DataManager.isWeapon(baseItem))  {
         maxItems = Yanfly.Param.ItemMaxWeapons;
      }
      if (DataManager.isArmor(baseItem))  {
         maxItems = Yanfly.Param.ItemMaxArmors;
      }        
      if (maxItems > 0) {
         if (baseItem.nonIndepdent === false) {
            if (DataManager.isItem(baseItem)) {
               baseItem = ( baseItem.baseItemId ? $dataItems[baseItem.baseItemId] : baseItem );
               var group = this.items();
            }
            if (DataManager.isWeapon(baseItem))  {
               baseItem = ( baseItem.baseItemId ? $dataWeapons[baseItem.baseItemId] : baseItem );
               var group = this.weapons();
            }
            if (DataManager.isArmor(baseItem))  {
               baseItem = ( baseItem.baseItemId ? $dataArmors[baseItem.baseItemId] : baseItem );
               var group = this.armors(); 
            }        
            var baseItemId = baseItem.id;
            for (var i = 0; i < group.length; ++i) {
               var item = group[i];
               if (!item) continue;
               if (!item.baseItemId) continue;
               if (item.baseItemId !== baseItemId) continue;
               amount += 1;
            }
            return amount;
         }
      }
   }
   if ( DataManager.isItem(baseItem) && $gameParty._items[baseItem.id]) amount = $gameParty._items[baseItem.id];
   if ( DataManager.isArmor(baseItem) && $gameParty._armors[baseItem.id]) amount = $gameParty._armors[baseItem.id];
   if ( DataManager.isWeapon(baseItem) && $gameParty._weapons[baseItem.id]) amount = $gameParty._weapons[baseItem.id];
   return amount;
}

Unco.AS.Game_BattlerBase_canPaySkillCost =
   Game_BattlerBase.prototype.canPaySkillCost;
Game_BattlerBase.prototype.canPaySkillCost = function(skill) {
   if (!this.canPaySkillItemAmmoCost(skill)) return false;
   return Unco.AS.Game_BattlerBase_canPaySkillCost.call(this, skill);
};

Game_BattlerBase.prototype.canPaySkillItemAmmoCost = function(skill) {
   var retVal = true;
   if (skill.id === this.attackSkillId()) {
      if (typeof this._equips !== 'undefined') {
         if (this._equips[0]._itemId > 0) {
            skill = $dataWeapons[ ( (typeof $dataWeapons[this._equips[0]._itemId].baseItemId === 'undefined') ? this._equips[0]._itemId : $dataWeapons[this._equips[0]._itemId].baseItemId ) ];
         }
      }
   }
	if (typeof skill.goldCost === 'number') {
	   if (skill.goldCost > $gameParty.gold()) {
         return false;
      }
	}
   for (var ammoId in skill.itemAmmoCost) {
      ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         var nbOwned = $gameParty.getItemAmount($dataItems[ammoId]);
         if (skill.itemAmmoCost[ammoId] > nbOwned) {
            return false;
         }
      }
   }
   for (var ammoId in skill.equipAmmoCost) {
      ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         if (this.hasArmor($dataArmors[ammoId])) {
				skill.equipAmmoCost[ammoId] = parseInt(skill.equipAmmoCost[ammoId]);
            if (!isNaN(skill.equipAmmoCost[ammoId])) {
               var nbOwned = 1+$gameParty.getItemAmount($dataArmors[ammoId]);
               if (skill.equipAmmoCost[ammoId] <= nbOwned) {
                  return true;
               }        
            } else retVal = false;
         } else retVal = false;
      }
   }
   return retVal;
};

Unco.AS.Game_BattlerBase_paySkillCost = Game_BattlerBase.prototype.paySkillCost;
Game_BattlerBase.prototype.paySkillCost = function(skill) {
    Unco.AS.Game_BattlerBase_paySkillCost.call(this, skill);
    this.paySkillItemAmmoCost(skill);
};

Game_BattlerBase.prototype.paySkillItemAmmoCost = function(skill) {
   if (skill.id === this.attackSkillId()) {
      if (typeof this._equips !== 'undefined') {
         if (this._equips[0]._itemId > 0) {
            skill = $dataWeapons[ ( (typeof $dataWeapons[this._equips[0]._itemId].baseItemId === 'undefined') ? this._equips[0]._itemId : $dataWeapons[this._equips[0]._itemId].baseItemId ) ];
         }
      }
   }
	if (typeof skill.goldCost === 'number') {
		$gameParty.loseGold(skill.goldCost);
	}
   for (var ammoId in skill.itemAmmoCost) {
      ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         $gameParty.gainItem($dataItems[ammoId],-skill.itemAmmoCost[ammoId]);
      }
   }
   for (var ammoId in skill.equipAmmoCost) {
      ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         skill.equipAmmoCost[ammoId] = parseInt(skill.equipAmmoCost[ammoId]);
         if (this.hasArmor($dataArmors[ammoId]) && !isNaN(skill.equipAmmoCost[ammoId]) && (skill.equipAmmoCost[ammoId] > 0)) {
            if ($gameParty.getItemAmount($dataArmors[ammoId]) > 0) {
               $gameParty.gainItem($dataArmors[ammoId],-Math.max(skill.equipAmmoCost[ammoId]-1,0));
            }
            if (  $gameParty.getItemAmount($dataArmors[ammoId]) === 0  ) {
					this._markForEquipDiscard = ammoId;
            } else {
               $gameParty.gainItem($dataArmors[ammoId],-1);
            }
         }
      }
   }
};


//=============================================================================
// BattleManager
//=============================================================================
Unco.AS.BattleManager_endAction = BattleManager.endAction;
BattleManager.endAction = function() {
   Unco.AS.BattleManager_endAction.call(this);
   if (typeof this._subject._markForEquipDiscard !== 'undefined') {
      for (var i in this._subject.equips()) {
         if ((typeof this._subject.equips()[i] !== 'undefined') && (this._subject.equips()[i] !== null)) {
            if (typeof this._subject.equips()[i].baseItemId !== 'undefined') {
               if (this._subject.equips()[i].baseItemId === this._subject._markForEquipDiscard) {
                  this._subject._markForEquipDiscard = this._subject.equips()[i].id;
               }
            }        
         }
      }
      this._subject.discardEquip($dataArmors[this._subject._markForEquipDiscard]);      
      delete this._subject._markForEquipDiscard;
   }
};

//=============================================================================
// Window_SkillList
//=============================================================================

if ((typeof Imported.YEP_SkillCore === 'undefined') || (Imported.YEP_SkillCore !== true)) {   
   Window_SkillList.prototype.drawSkillCost = function(skill, x, y, width) {
      var dw = width;
      if (this._actor.skillTpCost(skill) > 0) {
         this.changeTextColor(this.tpCostColor());
         var text = this._actor.skillTpCost(skill);
         this.drawText(text, x, y, dw, 'right');
         dw -= this.textWidth(text);
      }
      if (dw !== width) {
         var text = ' ';
         this.drawText(text, x, y, dw, 'right');
         dw -= this.textWidth(text);
      }
      if (this._actor.skillMpCost(skill) > 0) {
         this.changeTextColor(this.mpCostColor());
         var text = this._actor.skillMpCost(skill)
         this.drawText(text, x, y, dw, 'right');
         dw -= this.textWidth(text);
      } 
      return dw; 
   }  
}


Unco.AS.Window_SkillList_drawSkillCost = Window_SkillList.prototype.drawSkillCost;   
   
Window_SkillList.prototype.drawSkillCost = function(skill, wx, wy, width) {
   var dw = width;
   dw = this.drawAmmoCost(skill, wx, wy, dw);
   return Unco.AS.Window_SkillList_drawSkillCost.call(this,skill, wx, wy, dw);
};
   

Window_SkillList.prototype.drawAmmoCost = function(skill, wx, wy, dw) {
	if (typeof skill.goldCost === 'number') {		
      this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
      var text = String(skill.goldCost) + '\G';
      this.contents.fontSize = Unco.Param.ammoFontSize;
      this.drawText(text, wx, wy, dw, 'right');
      dw -= this.textWidth(text);
      this.resetFontSettings();
	}
   for (var ammoId in skill.itemAmmoCost) {
      ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
         var text = 'x' + String(skill.itemAmmoCost[ammoId]) + ( (Unco.Param.showAmmoLeft === 'false') ? "" : (  "/" + $gameParty.getItemAmount($dataItems[ammoId])  )  );
         this.contents.fontSize = Unco.Param.ammoFontSize;
         this.drawText(text, wx, wy, dw, 'right');
         dw -= this.textWidth(text);
         this.resetFontSettings();
         if ($dataItems[ammoId].iconIndex > 0) {
            var iw = wx + dw - Window_Base._iconWidth;
            this.drawIcon($dataItems[ammoId].iconIndex, iw, wy + 2);
            dw -= Window_Base._iconWidth + 2;
         }
      }
   }
   var zeroAmmo = false;
   for (var ammoId in skill.equipAmmoCost) {
      var ammoId = parseInt(ammoId);
      if (!isNaN(ammoId) && (ammoId > 0)) {
         if (this._actor.hasArmor($dataArmors[ammoId])) {
            this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
            var text = 'x' + String(skill.equipAmmoCost[ammoId]) + ( (Unco.Param.showAmmoLeft === 'false') ? "" : (  "/" + ( 1 + $gameParty.getItemAmount($dataArmors[ammoId]) )  )  );
            this.contents.fontSize = Unco.Param.ammoFontSize;
            this.drawText(text, wx, wy, dw, 'right');
            dw -= this.textWidth(text);
            this.resetFontSettings();
            if ($dataArmors[ammoId].iconIndex > 0) {
               var iw = wx + dw - Window_Base._iconWidth;
               this.drawIcon($dataArmors[ammoId].iconIndex, iw, wy + 2);
               dw -= Window_Base._iconWidth + 2;
            }
            zeroAmmo = false;
            break;
         } else {
            zeroAmmo = true;
            if (typeof smallestCost !== 'Number') {
               var smallestCost = skill.equipAmmoCost[ammoId];
            } else {
               if (skill.equipAmmoCost[ammoId] < smallestCost) smallestCost = skill.equipAmmoCost[ammoId];
            }
         }
      }
   }
   if (zeroAmmo === true) {
       this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
       var text = 'x' + String(smallestCost);
       this.contents.fontSize = Unco.Param.ammoFontSize;
       this.drawText(text, wx, wy, dw, 'right');
       dw -= this.textWidth(text);
       this.resetFontSettings();
       var iw = wx + dw - Window_Base._iconWidth;
       this.drawIcon( ( (skill.noAmmoIconId) ? skill.noAmmoIconId : Unco.Param.defaultNoAmmoIconId ) , iw, wy + 2);
       dw -= Window_Base._iconWidth + 2;
   }
   return dw;
};


//=============================================================================
// Window_Help
//=============================================================================

Window_Help.prototype.setItem = function(item) {
   var text = '';
   if (typeof item !== 'undefined') {
      if ((DataManager.isSkill(item)) && (Unco.Param.showAmmoLeftInDesc === 'true')) {
         var skill = item;
         var withAmmo = false;
	      if (typeof skill.goldCost === 'number') {
            text = text + $gameParty.gold() + '\\G';
            withAmmo = true;
			}
         for (var ammoId in skill.itemAmmoCost) {
            var cost = skill.itemAmmoCost[ammoId];
            ammoId = parseInt(ammoId);
            if (!isNaN(ammoId) && (ammoId > 0) && !isNaN(cost)) {
               var cost = parseInt(cost);
               var itemOwned = ($gameParty.getItemAmount($dataItems[ammoId]) > 0);
               text = text + '\\i[' + String($dataItems[ammoId].iconIndex) + ']' + ( ((cost <= 0) && itemOwned) ? '' : ( 'x' + String( $gameParty.getItemAmount($dataItems[ammoId]) ) )  );
               withAmmo = true;
            }
         }
         var zeroEquipAmmo = false;
         for (var ammoId in skill.equipAmmoCost) {
            var cost = skill.equipAmmoCost[ammoId];
            var ammoId = parseInt(ammoId);
            if (!isNaN(ammoId) && (ammoId > 0) && (typeof this._actor !== 'undefined')) {
               cost = parseInt(cost);
               withAmmo = true;
               if (this._actor.hasArmor($dataArmors[ammoId])) {
                  text = text + '\\i[' + String($dataArmors[ammoId].iconIndex) + ']' + ( (cost <= 0) ? '' : 'x' + String( 1 + $gameParty.getItemAmount($dataArmors[ammoId]) ) );
                  text = text + ( (typeof $dataArmors[ammoId].ammoDesc === 'undefined') ? '' : '(' + $dataArmors[ammoId].ammoDesc + ')' );
                  zeroEquipAmmo = false;
                  break;
               } else {
                  zeroEquipAmmo = true;
               }
            }
         }
         if (zeroEquipAmmo === true) text = text + '\\i[' + String( ( (skill.noAmmoIconId) ? skill.noAmmoIconId : Unco.Param.defaultNoAmmoIconId ) ) + ']x0';
         if (withAmmo === true) text = '\n' + ( (typeof skill.ammoText !== 'undefined') ? skill.ammoText : Unco.Param.descAmmoLeftText) + ' : ' + text;
      }
   }
   this.setText(item ? item.description + text : '');
};

Unco.AS.Window_SkillList_setHelpWindowItem = Window_SkillList.prototype.setHelpWindowItem;
Window_SkillList.prototype.setHelpWindowItem = function(item) {
   if (this._helpWindow) {
      this._helpWindow._actor = this._actor;
   }
   Unco.AS.Window_SkillList_setHelpWindowItem.call(this,item)
};

//=============================================================================
// Window_ActorCommand
//=============================================================================

Window_ActorCommand.prototype.getAmmoText = function(skill) {
    var ammoStr = '';  
    if (Unco.Param.showAmmoLeftForAttack === 'true') {
          for (var ammoId in skill.itemAmmoCost) {
             var ammoId = parseInt(ammoId);
             var ammoCost = parseInt(skill.itemAmmoCost[ammoId]);
             if (!isNaN(ammoId) && (ammoId > 0) && !isNaN(ammoCost) && (ammoCost > 0)) {
                if (ammoStr !== '') ammoStr += '|';
                ammoStr += String(  $gameParty.getItemAmount($dataItems[ammoId]) );               
             }
          }
          var zeroAmmo = false;
          for (var ammoId in skill.equipAmmoCost) {
             var ammoId = parseInt(ammoId);
             if (!isNaN(ammoId) && (ammoId > 0)) {
                if (this._actor.hasArmor($dataArmors[ammoId])) {
                   if (ammoStr !== '') ammoStr += '|';
                   ammoStr += String( 1+$gameParty.getItemAmount($dataArmors[ammoId]) );
                   zeroAmmo = false;
                   break;
                } else {
                   zeroAmmo = true;
                }
             }
          }
          if (zeroAmmo === true) {
             if (ammoStr !== '') ammoStr += '|';
             ammoStr += '0';
          }
    }
    if (ammoStr !== '') ammoStr = '[' + ammoStr + ']';
    return ammoStr;
};

Unco.AS.Window_ActorCommand_addCommand = Window_ActorCommand.prototype.addCommand;
if (Imported.BOB_BattleCommandList === true) {
   Window_ActorCommand.prototype.addCommand = function(name, symbol, enabled, ext, icon) {
      if (symbol === 'attack') {
         var ammoStr = this.getAmmoText($dataWeapons[this._actor._equips[0]._itemId]);
         name = name + ammoStr;
      }
      if (symbol === 'customSkill') {
         var ammoStr = this.getAmmoText(ext);
         name = name + ammoStr;
      }
      Unco.AS.Window_ActorCommand_addCommand.call(this,name, symbol, enabled, ext, icon);
   };
} else {
   Window_ActorCommand.prototype.addCommand = function(name, symbol, enabled, ext) {
      if (symbol === 'attack') {
         var ammoStr = this.getAmmoText($dataWeapons[this._actor._equips[0]._itemId]);
         name = name + ammoStr;
      }
      Unco.AS.Window_ActorCommand_addCommand.call(this,name, symbol, enabled, ext);
   };
}

Unco.AS.Window_ActorCommand_setHelpWindowItem = Window_ActorCommand.prototype.setHelpWindowItem;
Window_ActorCommand.prototype.setHelpWindowItem = function(item) {
   if (this._helpWindow && this._actor) {
      this._helpWindow._actor = this._actor;
   }
   Unco.AS.Window_ActorCommand_setHelpWindowItem.call(this,item)
};



if (Imported.YEP_SkillCore === true) {
//=============================================================================
// Window_Base
//=============================================================================
   Window_Base.prototype.mustDrawActorAmmo = function(actor) {
      if (typeof actor._equips !== 'undefined') {
         if (actor._equips[0]._itemId > 0) {
            skill = $dataWeapons[actor._equips[0]._itemId];
            for (var ammoId in skill.itemAmmoCost) {
               var ammoId = parseInt(ammoId);
               if (!isNaN(ammoId) && (ammoId > 0)) {                
                  if (!isNaN(skill.itemAmmoCost[ammoId])) {
                     return true;
                  }
               }
            }
            for (var ammoId in skill.equipAmmoCost) {
               var ammoId = parseInt(ammoId);
               if (!isNaN(ammoId) && (ammoId > 0)) {                   
                  if (!isNaN(skill.equipAmmoCost[ammoId])) {
                     return true;
                  }
               }
           }
         }
      }
      return false;
   }
   Window_Base.prototype.drawActorAmmo = function(actor, x, y, width) {
      if (typeof actor._equips !== 'undefined') {
         if (actor._equips[0]._itemId > 0) {
            skill = $dataWeapons[ ( (typeof $dataWeapons[actor._equips[0]._itemId].baseItemId === 'undefined') ? actor._equips[0]._itemId : $dataWeapons[actor._equips[0]._itemId].baseItemId ) ];
            var dw = width;
            for (var ammoId in skill.itemAmmoCost) {
               var ammoId = parseInt(ammoId);
               if (!isNaN(ammoId) && (ammoId > 0)) { 
                  var ammoCost = parseInt(skill.itemAmmoCost[ammoId]);               
                  this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
                  var text = (ammoCost <= 0) ? '' : 'x' + String( $gameParty.getItemAmount($dataItems[ammoId]) );
                  this.contents.fontSize = Unco.Param.ammoFontSize;
                  this.drawText(text, x, y, dw, 'right');
                  dw -= this.textWidth(text);
                  this.resetFontSettings();
                  if ($dataItems[ammoId].iconIndex > 0) {
                     var iw = x + dw - Window_Base._iconWidth;
                     this.drawIcon($dataItems[ammoId].iconIndex, iw, y + 2);
                     dw -= Window_Base._iconWidth + 2;
                  }
               }
            }
            var zeroAmmo = false;
            for (var ammoId in skill.equipAmmoCost) {
               var ammoId = parseInt(ammoId);
               if (!isNaN(ammoId) && (ammoId > 0)) {
                  var ammoCost = parseInt(skill.equipAmmoCost[ammoId]);  
                  if (actor.hasArmor($dataArmors[ammoId])) {
                     this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
                     var text = (ammoCost <= 0) ? '' : 'x' + String( 1+$gameParty.getItemAmount($dataArmors[ammoId]) );
                     this.contents.fontSize = Unco.Param.ammoFontSize;
                     this.drawText(text, x, y, dw, 'right');
                     dw -= this.textWidth(text);
                     this.resetFontSettings();
                     if ($dataArmors[ammoId].iconIndex > 0) {
                        var iw = x + dw - Window_Base._iconWidth;
                        this.drawIcon($dataArmors[ammoId].iconIndex, iw, y + 2);
                        dw -= Window_Base._iconWidth + 2;
                     }
                     zeroAmmo = false;
                     break;
                  } else {
                     zeroAmmo = true;
                  }
               }
            }
            if (zeroAmmo === true) {
               this.changeTextColor(this.textColor(Unco.Param.ammoFontColor));
               var text = 'x0';
               this.contents.fontSize = Unco.Param.ammoFontSize;
               this.drawText(text, x, y, dw, 'right');
               dw -= this.textWidth(text);
               this.resetFontSettings();
               var iw = x + dw - Window_Base._iconWidth;
               this.drawIcon( ( (skill.noAmmoIconId) ? skill.noAmmoIconId : Unco.Param.defaultNoAmmoIconId ) , iw, y + 2);
               dw -= Window_Base._iconWidth + 2;
            }
         }
      }
   };
   
//=============================================================================
// DataManager (again)
//=============================================================================
   Unco.AS.DataManager_processGSCNotetags1 = DataManager.processGSCNotetags1;
   DataManager.processGSCNotetags1 = function(group) {
     Unco.AS.DataManager_processGSCNotetags1.call(this,group);
     for (var n = 1; n < group.length; n++) {
       var obj = group[n];
       var notedata = obj.note.split(/[\r\n]+/);

       for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:SWAP GAUGE|gauge)[ ](\d+):[ ](.*)>/i)) {
           var gauge = parseInt(RegExp.$1);
           var text = String(RegExp.$2).toUpperCase();
           if (['AMMO'].contains(text)) {
             if (gauge === 1) obj.gauge1 = text;
             if (gauge === 2) obj.gauge2 = text;
             if (gauge === 3) obj.gauge3 = text;
           }
         }
       }
     }
   };
   Unco.AS.DataManager_processGSCNotetags2 = DataManager.processGSCNotetags2;
   DataManager.processGSCNotetags2 = function(group) {
     Unco.AS.DataManager_processGSCNotetags2.call(this,group);
     for (var n = 1; n < group.length; n++) {
       var obj = group[n];
       var notedata = obj.note.split(/[\r\n]+/);

       for (var i = 0; i < notedata.length; i++) {
         var line = notedata[i];
         if (line.match(/<(?:SWAP GAUGE|gauge)[ ](\d+):[ ](.*)>/i)) {
           var gauge = parseInt(RegExp.$1);
           var text = String(RegExp.$2).toUpperCase();
           if (['AMMO'].contains(text)) {
             if (gauge === 1) obj.gauge1 = text;
             if (gauge === 2) obj.gauge2 = text;
             if (gauge === 3) obj.gauge3 = text;
           }
         }
       }
     }
   };
   
   Unco.AS.Window_Window_Base_drawActorHp = Window_Base.prototype.drawActorHp;
   Window_Base.prototype.drawActorHp = function(actor, x, y, width) {
       if ((this.mustDrawActorAmmo(actor)) && (actor.gauge1() === 'AMMO')) {
       this.drawActorAmmo(actor, x, y, width);
       } else {
         Unco.AS.Window_Window_Base_drawActorHp.call(this, actor, x, y, width);
       }
   };
   
   Unco.AS.Window_Window_Base_drawActorMp = Window_Base.prototype.drawActorMp;
   Window_Base.prototype.drawActorMp = function(actor, x, y, width) {
       if ((this.mustDrawActorAmmo(actor)) && (actor.gauge2() === 'AMMO')) {
         this.drawActorAmmo(actor, x, y, width);
       } else {
         Unco.AS.Window_Window_Base_drawActorMp.call(this, actor, x, y, width);
       }
   };
   
   Unco.AS.Window_Window_Base_drawActorTp = Window_Base.prototype.drawActorTp;
   Window_Base.prototype.drawActorTp = function(actor, x, y, width) {
       if ((this.mustDrawActorAmmo(actor)) && (actor.gauge3() === 'AMMO')) {
       this.drawActorAmmo(actor, x, y, width);
       } else {
         Unco.AS.Window_Window_Base_drawActorTp.call(this, actor, x, y, width);
       }
   };
}


//-----------------------------------------------------------------------------
// Window_Ammo
//
// The window for displaying the ammunition amount for attack action.

function Window_Ammo() {
    this.initialize.apply(this, arguments);
}

Window_Ammo.prototype = Object.create(Window_Base.prototype);
Window_Ammo.prototype.constructor = Window_Ammo;

Window_Ammo.prototype.initialize = function(numLines) {
    var width = Graphics.boxWidth;
    var height = this.fittingHeight(1);
    var y = Graphics.height-this.fittingHeight(6);
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this._text = '';
};

Window_Ammo.prototype.setText = function(text) {
    if (this._text !== text) {
        this._text = text;
        this.refresh();
    }
};

Window_Ammo.prototype.clear = function() {
    this.setText('');
};


Window_Ammo.prototype.setItem = function(actor,item) {
   var text = '';
   if (typeof item !== 'undefined') {
      if ((DataManager.isSkill(item)) && (Unco.Param.showAmmoLeftInDesc === 'true')) {
         var skill = item;
         if (skill.id === actor.attackSkillId()) {
            if (typeof actor._equips !== 'undefined') {
               if (actor._equips[0]._itemId > 0) {
                  skill = $dataWeapons[ ( (typeof $dataWeapons[actor._equips[0]._itemId].baseItemId === 'undefined') ? actor._equips[0]._itemId : $dataWeapons[actor._equips[0]._itemId].baseItemId ) ];
               }
            }
         }
         var withAmmo = false;

         for (var ammoId in skill.itemAmmoCost) {
            var cost = skill.itemAmmoCost[ammoId];
            ammoId = parseInt(ammoId);
            if (!isNaN(ammoId) && (ammoId > 0) && !isNaN(cost)) {
               var cost = parseInt(cost);
               var itemOwned = ($gameParty.getItemAmount($dataItems[ammoId]) > 0);
               text = text + '\\i[' + String($dataItems[ammoId].iconIndex) + ']' + ( ((cost <= 0) && itemOwned) ? '' : ( 'x' + String( $gameParty.getItemAmount($dataItems[ammoId]) ) )  );
               withAmmo = true;
            }
         }
         var zeroEquipAmmo = false;
         for (var ammoId in skill.equipAmmoCost) {
            var cost = skill.equipAmmoCost[ammoId];
            var ammoId = parseInt(ammoId);
            if (!isNaN(ammoId) && (ammoId > 0)) {
               cost = parseInt(cost);
               withAmmo = true;
               if (actor.hasArmor($dataArmors[ammoId])) {
                  text = text + '\\i[' + String($dataArmors[ammoId].iconIndex) + ']' + ( (cost <= 0) ? '' : 'x' + String( 1 + $gameParty.getItemAmount($dataArmors[ammoId]) )  );
                  text = text + ( (typeof $dataArmors[ammoId].ammoDesc === 'undefined') ? '' : '(' + $dataArmors[ammoId].ammoDesc + ')' );
                  zeroEquipAmmo = false;
                  break;
               } else {
                  zeroEquipAmmo = true;
               }
            }
         }
         if (zeroEquipAmmo === true) text = text + '\\i[' + String( ( (skill.noAmmoIconId) ? skill.noAmmoIconId : Unco.Param.defaultNoAmmoIconId ) ) + ']x0';
         if (withAmmo === true) text = ( (typeof skill.ammoText !== 'undefined') ? skill.ammoText : Unco.Param.descAmmoLeftText) + ' : ' + text;
      }
   }
   if (text !== '') {
      this.setText(item ? text : '');
      this.show();
   }
};

Window_Ammo.prototype.refresh = function() {
    this.contents.clear();
    this.drawTextEx(this._text, this.textPadding(), 0);
};

Unco.AS.Scene_Battle_createAllWindows = Scene_Battle.prototype.createAllWindows;
Scene_Battle.prototype.createAllWindows = function() {
    Unco.AS.Scene_Battle_createAllWindows.call(this);
    this.createAmmoWindow();
};

Scene_Battle.prototype.createAmmoWindow = function() {
    this._ammoWindow = new Window_Ammo();
    this._ammoWindow.visible = false;
    this.addWindow(this._ammoWindow);
};

Scene_Battle.prototype.showAmmoWindow = function() {
  if (Unco.Param.showAmmoLeftActorCommand === 'true') this._ammoWindow.setItem(BattleManager.actor(),$dataSkills[BattleManager.actor().attackSkillId()]);
};

Unco.AS.Scene_Battle_onSelectAction = Scene_Battle.prototype.onSelectAction;
Scene_Battle.prototype.onSelectAction = function() {
    this._ammoWindow.hide();
    Unco.AS.Scene_Battle_onSelectAction.call(this);
};

Unco.AS.Scene_Battle_commandAttack = Scene_Battle.prototype.commandAttack;
Scene_Battle.prototype.commandAttack = function() {
   this._ammoWindow.hide();
   Unco.AS.Scene_Battle_commandAttack.call(this);
};
Unco.AS.Scene_Battle_commandSkill = Scene_Battle.prototype.commandSkill;
Scene_Battle.prototype.commandSkill = function() {
   this._ammoWindow.hide();
   Unco.AS.Scene_Battle_commandSkill.call(this);
};
Unco.AS.Scene_Battle_commandGuard = Scene_Battle.prototype.commandGuard;
Scene_Battle.prototype.commandGuard = function() {
   this._ammoWindow.hide();
   Unco.AS.Scene_Battle_commandGuard.call(this);
};
Unco.AS.Scene_Battle_commandItem = Scene_Battle.prototype.commandItem;
Scene_Battle.prototype.commandItem = function() {
   this._ammoWindow.hide();
   Unco.AS.Scene_Battle_commandItem.call(this);
};
Unco.AS.Scene_Battle_startActorCommandSelection = Scene_Battle.prototype.startActorCommandSelection;
Scene_Battle.prototype.startActorCommandSelection = function() {
   this.showAmmoWindow();
   Unco.AS.Scene_Battle_startActorCommandSelection.call(this);
};
Unco.AS.Scene_Battle_onSkillCancel = Scene_Battle.prototype.onSkillCancel;
Scene_Battle.prototype.onSkillCancel = function() {
   this.showAmmoWindow();
   Unco.AS.Scene_Battle_onSkillCancel.call(this);
};
Unco.AS.Scene_Battle_onItemCancel = Scene_Battle.prototype.onItemCancel;
Scene_Battle.prototype.onItemCancel = function() {
   this.showAmmoWindow();
   Unco.AS.Scene_Battle_onItemCancel.call(this);
};
Unco.AS.Scene_Battle_onEnemyCancel = Scene_Battle.prototype.onEnemyCancel;
Scene_Battle.prototype.onEnemyCancel = function() {
   if (this._actorCommandWindow.currentSymbol() === 'attack') {
      this.showAmmoWindow();
   }
   Unco.AS.Scene_Battle_onEnemyCancel.call(this);
};